@@ -47,14 +47,18 @@
 #undef MIN
 #undef MAX
 
-extern gint camel_verbose_debug;
-
-#define d(x) (camel_verbose_debug ? (x) : 0)
+#define d(x) (camel_debug ("smtp") ? (x) : 0)
 
 /* Specified in RFC 821 */
 #define SMTP_PORT  25
 #define SMTPS_PORT 465
 
+enum {
+	PROP_0,
+	PROP_CONNECTABLE,
+	PROP_HOST_REACHABLE
+};
+
 /* support prototypes */
 static GHashTable *	esmtp_get_authtypes	(const guchar *buffer);
 static gboolean		smtp_helo		(CamelSmtpTransport *transport,
@@ -180,9 +184,6 @@ connect_to_server (CamelService *service,
 		transport->flags &= ~CAMEL_SMTP_TRANSPORT_IS_ESMTP;
 
 		if (!smtp_helo (transport, cancellable, error)) {
-			camel_service_disconnect_sync (
-				CAMEL_SERVICE (transport),
-				TRUE, cancellable, NULL);
 			success = FALSE;
 			goto exit;
 		}
@@ -246,9 +247,6 @@ connect_to_server (CamelService *service,
 	/* We are supposed to re-EHLO after a successful STARTTLS to
 	 * re-fetch any supported extensions. */
 	if (!smtp_helo (transport, cancellable, error)) {
-		camel_service_disconnect_sync (
-			CAMEL_SERVICE (transport),
-			TRUE, cancellable, NULL);
 		success = FALSE;
 	}
 
@@ -262,13 +260,14 @@ exception_cleanup:
 	g_object_unref (transport->ostream);
 	transport->ostream = NULL;
 
-	transport->connected = FALSE;
-
 	success = FALSE;
 
 exit:
 	g_free (host);
 
+	if (!success)
+		transport->connected = FALSE;
+
 	return success;
 }
 
@@ -280,6 +279,48 @@ authtypes_free (gpointer key,
 	g_free (value);
 }
 
+static void
+smtp_transport_set_property (GObject *object,
+                             guint property_id,
+                             const GValue *value,
+                             GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_CONNECTABLE:
+			camel_network_service_set_connectable (
+				CAMEL_NETWORK_SERVICE (object),
+				g_value_get_object (value));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+smtp_transport_get_property (GObject *object,
+                             guint property_id,
+                             GValue *value,
+                             GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_CONNECTABLE:
+			g_value_take_object (
+				value,
+				camel_network_service_ref_connectable (
+				CAMEL_NETWORK_SERVICE (object)));
+			return;
+
+		case PROP_HOST_REACHABLE:
+			g_value_set_boolean (
+				value,
+				camel_network_service_get_host_reachable (
+				CAMEL_NETWORK_SERVICE (object)));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
 static gchar *
 smtp_transport_get_name (CamelService *service,
                          gboolean brief)
@@ -383,8 +424,7 @@ smtp_transport_connect_sync (CamelService *service,
 		g_object_unref (session);
 
 		if (!success)
-			camel_service_disconnect_sync (
-				service, TRUE, cancellable, NULL);
+			transport->connected = FALSE;
 	}
 
 exit:
@@ -770,9 +810,14 @@ smtp_transport_get_default_port (CamelNetworkService *service,
 static void
 camel_smtp_transport_class_init (CamelSmtpTransportClass *class)
 {
+	GObjectClass *object_class;
 	CamelServiceClass *service_class;
 	CamelTransportClass *transport_class;
 
+	object_class = G_OBJECT_CLASS (class);
+	object_class->set_property = smtp_transport_set_property;
+	object_class->get_property = smtp_transport_get_property;
+
 	service_class = CAMEL_SERVICE_CLASS (class);
 	service_class->settings_type = CAMEL_TYPE_SMTP_SETTINGS;
 	service_class->get_name = smtp_transport_get_name;
@@ -783,6 +828,18 @@ camel_smtp_transport_class_init (CamelSmtpTransportClass *class)
 
 	transport_class = CAMEL_TRANSPORT_CLASS (class);
 	transport_class->send_to_sync = smtp_transport_send_to_sync;
+
+	/* Inherited from CamelNetworkService. */
+	g_object_class_override_property (
+		object_class,
+		PROP_CONNECTABLE,
+		"connectable");
+
+	/* Inherited from CamelNetworkService. */
+	g_object_class_override_property (
+		object_class,
+		PROP_HOST_REACHABLE,
+		"host-reachable");
 }
 
 static void
@@ -1140,10 +1197,6 @@ smtp_helo (CamelSmtpTransport *transport,
 		g_prefix_error (error, _("HELO command failed: "));
 		camel_operation_pop_message (cancellable);
 
-		camel_service_disconnect_sync (
-			CAMEL_SERVICE (transport),
-			FALSE, cancellable, NULL);
-
 		return FALSE;
 	}
 	g_free (cmdbuf);
